package POE::Helper::Leve::patternList;
##############################################################################
=pod

=head1 NAME

patternList

=head1 DESCRIPTION

a module to keep record of files having a numbering, e.g. episodes
patternList  a module to keep record of files having a numbering, e.g. episodes
of some series

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 patternList.pm is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

=head3 global stuff

Some global stuff is defined here

=cut

######################################################################
use strict;
use Data::Dumper;

use vars qw/$VERSION/;

$VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
##############################################################################
=pod

=head1 Methods of this class

=over

=item init

set up the main datastructures of this object, make it ready for the run

=cut

######################################################################
sub init
{
  my($this,$settings,$reporter) = @_;
  die("wrong type for settings: ".ref($settings)."\n") if(ref($settings) ne 'HASH');

  $this->{"wishlist"} = [];	#list of regexps that the user wants to fetch 
    $this->{"series"} = {};	#list of regexps and serial numbers that the user wants to fetch 
    $this->{"settings"} = $settings;#global settings where this object might fetch some stuff

    $this->load();
  #print("dumping settings: ".Dumper($settings)."\n");
  $this->{"reporter"} = $reporter;
}#sub init
######################################################################
=pod

=item new 

constructor for the new xchat handler object

=cut

######################################################################
sub new 
{
  my $fs = shift;
  my $serverName = shift;
  my $o_xchatHandler = { };

  bless $o_xchatHandler, "POE::Helper::Leve::patternList";
  return $o_xchatHandler;
}# sub new 
##############################################################################
=pod

=item match2pattern

compare the incoming filename to the pattern list, and return the 
pattern on success, the modus tells us if we are in dirscan modus or file
transfer success modus, in the latter case we remove or update the pattern
info

=cut

######################################################################
sub match2pattern
{
  my ($this, $args) = @_;
  my $line = $args->{"line"};
  #print("match2pattern parsing exile: $line\n") if($line =~ /last.*?exile/i);
  $line = lc($line);
  my $modus = $args->{"mode"};
  #check if this matches some of our patterns
  my $pattcounter = 0;
  #some servers leave http type spaces in the names, remove them
  $line =~ s/%20/ /g;
  #foreach my $pattern (@::wishlist)
  #print("considering line $line\n");
  #check if a word that should be avoided is in the line
  #print("Forbidden words: ".$this->{"settings"}->{"avoid"}."\n");
  my $forbidden;
  @$forbidden = split(/,/,$this->{"settings"}->{"avoid"});
  #print("Forbidden words: ".Dumper($forbidden)."\n");
  foreach my $pattern (@$forbidden)
  {
    if($line =~ /$pattern/i)
    {
      #print("IGNORING $line forbidden word: $pattern\n");
      return; 
    }
  }# foreach my $pattern (@$forbidden)

  foreach my $pattern (@{$this->{"wishlist"}})
  {
    if($line =~ /$pattern/i)
    {
      #Yoho and a bottle of Rhum!
      if($modus)
      {
	my $removedwish = $this->delwish($pattcounter);
	return $removedwish; 
      }#if($modus)
      #print("match2pattern return wish: $pattern\n");
      return $pattern;
    }#if($line =~ /$pattern/i)
    $pattcounter++;
  }#foreach my $pattern (@::wishlist)
  #sort in reverse order, that way more complicated patterns are tryed before more simple ones, allowing discirmination of v1 and v2 series e.g.
  my $series = $this->{"series"};
  foreach my $pattern (sort {$b cmp $a}(keys(%$series)))
  {
    my $case = $this->getSerie($pattern);
    my $min = $case->{"min"};
    my $max = $case->{"max"};
    my $fetched = $case->{"intermediate"};
    if($line =~ /$pattern.*?(\d+)/i)
    {
      my $episode = int($1);# force this type to be numeric
	#print("match2pattern parsing exile with : $line $episode\n") if($line =~ /last.*?exile/i);
       #$this->{"reporter"}->print_debug("match $pattern.*$episode");
      if($modus)
      {
	$this->seriesAdvance($pattern,$episode);
       $this->{"reporter"}->print_debug("advanced series returning $pattern*$episode");
	print(" $line corresponds to $pattern advancing series\n");
	return $pattern.'.*?'.$episode;
      }#if($modus)
      #fetching is ok if the eps is higher thant the minimum and if we haven't
      #it yet
      if($episode >= $min && !($fetched->{$episode}) && $episode <= $max)
      {
       $this->{"reporter"}->print_debug("$pattern*$episode in boundaries> $min\n");# if($line =~ /exile/i);
	$args->{"pattern"} = $pattern.'.*?'.$episode;
	#print(" $line corresponds to $pattern and is to be fetched\n");
	return $args;
      }#if($episode >= $min)
      #print("end match2pattern for exile $line\n") if($line =~ /last.*?exile/i);
      #print(" $line corresponds to $pattern but is allready fetched\n");
    }#if($line =~ /$pattern/i)
  }#foreach my $pattern (@::wishlist)
  #print("$line is not a monitored series\n");

  return;
}#sub match2pattern
######################################################################
=pod

=item load

loading all the relevant stuff

=cut

######################################################################
sub load
{
  my($this) = @_;

  my $filename = $this->{"settings"}->{"seriesFile"};
  $filename = "$ENV{HOME}/.leve/leve.series" if !$filename;
  #print("dumping settings: ".Dumper($settings)."\n");
  print("loading series from $filename\n");
  my $msg = $this->loadSeries($filename);
  $filename = $this->{"settings"}->{"wishlist"};
  $filename = "$ENV{HOME}/.leve/leve.wishes" if !$filename;
  $msg .= $this->loadWishes($filename);

  return "$msg\n";
}#sub load
######################################################################
=pod

=item loadWishes

loading the wishes database

=cut

######################################################################
sub loadWishes
{
  my($this,$filename) = @_;

  my $wishlist = $this->{"wishlist"};
  if(open(STATFILE,$filename))
  {
    while(<STATFILE>)
    {
      chomp;
      $_ =  $this->patternrevision($_);
      my $actPat = $_;
      push( @$wishlist,$_) if(!(grep(/$actPat/,@$wishlist)));
    }#while(<STATFILE>)
    close(STATFILE);
  }# if(open(STATFILE,">$ENV{HOME}/.leve/leve.state")) 
  return "LEVE: loaded wishes from $filename";
}#sub loadWishes
######################################################################
=pod

=item loadSeries

load a file first col is the series pattern, rest are maxto fetch,
  min-fetched, and allready fetched numbers, if max is set to 0 fetching is
  unlimited, beware no spaces in the pattern!

=cut

######################################################################
sub loadSeries
{
  my($this,$filename) = @_;
# restoring the wishes database
    my $patternNames = join(' ',keys(%{$this->{"series"}}));
    $patternNames .= ' ';
  if(open(STATFILE,$filename)) 
  {
    my $ignore = 0;
    while(my $line = <STATFILE>)
    {
      chomp $line;
      my $pattern = "";

      if(index($line,'#') >=0 )
      {
	$line = substr($line,0,index($line,'#'));
      }# if(index($line,'#') >=0 )
      $line = "" if($line =~ /<\?/);
      $ignore = 1 if($line =~ /<!--/);
      $ignore = 0 if($line =~ /-->/);

      #catch new style xmls type, first name is the env name
      $line = "type=\"$1\" $2" if($line =~ /^\s*<\s*(\S+)\s+(.*)$/);
      $line = $1 if($line =~ /^(.*?)\\>\s*$/);

      if(!$ignore && $line && $line =~ /=/)
      {
	#we have a line with arguments
	#clean up to have a consistent state
	$line =~ s/\s+=/=/g;
	$line =~ s/=\s+/=/g;
	my @matches = ($line =~ /(\S+=".*?")/g);
	my $entry = {};

	foreach my $field (@matches)
	{
	  my($key,$value) = split(/=/,$field);
	  $value =~ s/^"//;
	  $value =~ s/"$//;
	  if($key =~ /series/i)
	  {
	    #old style conf
	    $key = "name";
	  }# if($key =~ /series/i)
	  elsif($key =~ /fetched/i)
	  {
	    $key = "intermediate";

	    my $subfetched = {};
	    foreach my $i (split(' ',$value))
	    {
	      $i = int($i); #force numerical interpretation
		$subfetched->{$i} = 1;
	    }#foreach my $i (split(' ',$loaded))
	    $value = $subfetched;
	  }# elsif($key =~ /fetched/i)
	  $entry->{$key} = $value;
	}# foreach my $field (@matches)


	$pattern =  $this->patternrevision($entry->{"name"});
	$entry->{"priority"} = 0 if !$entry->{"priority"};
	delete $entry->{"name"};

	if($pattern)
	{
	  if(!($this->{"series"}->{$pattern}))
	  {
	    $this->{"series"}->{$pattern} = $entry;
	  }
	  else
	  {
	    #the given series is allready in our base, check for differences
	    if(!($entry->{"type"} && $entry->{"type"} eq "alias"))
	    {
	      $this->{"series"}->{$pattern}->{"min"} = $entry->{"min"} 
	      if($this->{"series"}->{$pattern}->{"min"} < $entry->{"min"});

	      $this->{"series"}->{$pattern}->{"max"} = $entry->{"max"} 
	      if($this->{"series"}->{$pattern}->{"max"} < $entry->{"max"});

	      $this->{"series"}->{$pattern}->{"priority"} = $entry->{"priority"} 
	      if($this->{"series"}->{$pattern}->{"priority"} != $entry->{"priority"});

	      foreach my $subEpisode (keys(%{$entry->{"intermediate"}}))
	      {
		if(!($this->{"series"}->{$pattern}->{"intermediate"}->{$subEpisode}))
		{
		  $this->{"series"}->{$pattern}->{"intermediate"}->{$subEpisode} = 1;
		}# if(!($series{$pattern}->{"intermediate"}->{$subEpisode}))
	      }# foreach my $subEpisode (keys(%{$fetched{"intermediate"}}))
	    }#if(!($type && $type eq "alias"))
	  }# else
	}#if($pattern)
	#remove the actual pattern from our list
	$patternNames =~ s/\Q$pattern\E //;

      }# if($line && $line =~ /=/)

    }#while(<STATFILE>)

    close(STATFILE);

    $patternNames =~ s/\s+/ /g;
    my @obsoletePatterns = split(/ /,$patternNames);
    foreach my $toRemove (@obsoletePatterns)
    {
      $this->delSerie($toRemove);
      #delete $this->{"series"}->{$toRemove};
    }# foreach my $toRemove (@obsoletePatterns)
  }# if(open(STATFILE,">$ENV{HOME}/.leve/leve.state")) 

  return "LEVE: loaded series";
}#sub loadSeries
######################################################################
=pod

=item save

Save the list of series and the whishlist!

needs the settings:

settings->{"seriesFile"}; default: $ENV{HOME}/.leve/leve.series
settings->{"whishesFile"}; default: $ENV{HOME}/.leve/leve.wishes

=cut

######################################################################
sub save
{
  #my($this) = @_;
  my ($this) = @_;
  my $msg;

  my $filename = $this->{"settings"}->{"seriesFile"};
  $filename = "$ENV{HOME}/.leve/leve.series" if !$filename;

  if(open(STATFILE,">$filename")) 
  {
    print STATFILE "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
    print STATFILE "<catalog>\n";


    my $series = $this->{"series"};
    foreach my $pattern (sort(keys(%$series)))
    {
      my $case = $series->{$pattern};
      #print("Saving entry ".Dumper($case )."\n");
      my $msg = "<";

      if($case->{"type"} && $case->{"type"} eq "alias")
      {
	$msg .= "alias name=\"".$pattern."\" ref=\"".$case->{"ref"}."\"";
      }#if(!($case->{"type"} && $case->{"type"} eq "alias"))
      else
      {
	$case->{"tag"} = " " if(!($case->{"tag"}));
	$case->{"priority"} = 0 if(!($case->{"priority"}));
	my $fetched = $series->{$pattern}{"intermediate"};
	$msg = "<series name=\"$pattern\" max=\"".$case->{"max"}."\" min=\"";
	$msg .= $case->{"min"}."\" priority=\"".$case->{"priority"}."\"";
	$msg .= " fetched=\"".join(' ',sort{ $b <=> $a }(keys(%$fetched)))."\"";
	$msg .= " tag=\"".$case->{"tag"}."\"";
      }#else
      $msg .= "/>\n";

      print STATFILE $msg;
    }
    print STATFILE "</catalog>\n";
    close(STATFILE);
    $msg .= "successfully stored series into $filename\n";
  }# if(open(STATFILE,">$ENV{HOME}/.leve/leve.state")) 

  $filename = $this->{"settings"}->{"whishesFile"};
  $filename = "$ENV{HOME}/.leve/leve.wishes" if !$filename;
  my $wishlist = $this->{"wishlist"};
  if(open(STATFILE,">$filename")) 
  {
    foreach my $pattern (@$wishlist)
    {
      print STATFILE "$pattern\n";
    }
    close(STATFILE);
    $msg .= "successfully stored wishes into $filename\n";
  }# if(open(STATFILE,">$ENV{HOME}/.leve/leve.state")) 
  return($msg);
}#sub saveSeries
######################################################################
=pod

=item delwish

delete pattern no

=cut

######################################################################
sub delwish
{
  my($this,$position) = @_;

  my $msg ;
  if($position =~ /\d+/)
  {
    my $wishlist = $this->{"wishlist"};
    my $removed = splice(@$wishlist,$position,1);
    $msg .= "removed pat: '$removed'";
    return $removed;
  }
  else
  {
    $position =  $this->patternrevision($position);
    my $series = $this->{"series"};
    if($series->{$position})
    {
      $msg .= "removed series '$position'";
      delete $series->{$position};
    }#if($series->{$position})
    else
    {
      $msg .= "Error: provide a valid pattern index";
    }
  }
  return($msg);
}#sub delwish
######################################################################
=pod

=item seriesAdvance

check series boundaries and adjust them if needed

=cut

######################################################################
sub seriesAdvance
{
  my ($this,$pattern,$episode) = @_;

  my $case =  $this->getSerie($pattern);
  print("seriesAdvance for $pattern with data=".Dumper($case)."\n");
  my $min = $case->{"min"};
  my $max = $case->{"max"};
  my $fetched = $case->{"intermediate"};

  if ($episode == $min)
  {
    $min++;
    #roll up the allready fetched eps up to next gap
    while($fetched->{$min})
    {
      delete $fetched->{$min};
      $min++;
    }#while($case->{$min})

    #my $series = $this->{"series"};
    $case->{"min"} = $min;
    if($case->{"min"} > $max)
    {
      #print_log("removing series pattern:".$series->{$pattern});
      $this->delSerie($pattern);
    }# if($case->{"min"} > $max)
  }#if ($episode == $min+1)
  else
  {
    $fetched->{$episode} = 1 if($episode > $min);
  }
}#sub seriesAdvance
######################################################################
=pod

=item patternrevision

switch patter active chars to perlish and non greedy patterns

=cut

######################################################################
sub patternrevision
{
  my ($this,$line) = @_;
  #change * into .*?
  $line =~ s/(\w)\*/$1.*?/g;
  #change .* into .*?
  $line =~ s/\.\*(?!\?)/.*?/g;
  return $line;
}#sub patternrevision
######################################################################
=pod

=item series

list/add series

=cut

######################################################################
sub series
{
  my  ($this,$line) = @_;

  my $msg;
  if($line)
  {
    my ($pattern,$max,$min,$rest) = split(' ',$line,4);
    $pattern =  $this->patternrevision($pattern);
    $min = 1 if(!$min || $min < 0);
    $max = 1023 if(!$max || $max < $min);
    my %fetched = ();
    $fetched{"min"} = $min;
    $fetched{"max"} = $max;
    my %subfetched = ();
    $fetched{"intermediate"} =  \%subfetched;
    if($rest)
    {
      foreach my $i (split(' ',$rest))
      {
	$subfetched{$i} = 1;
      }#foreach my $i (split(' ',$loaded))
    }#if($rest)
    my $series = $this->{"series"};
    $series->{$pattern} = \%fetched;
    $rest = "NONE" if(!$rest);
    $msg .= "LEVE: added '$pattern' to series with $min-$max skip:$rest";
  }# if($pattern)
  else
  {
    my $index = 0;
    $msg .= "---- Series ----\n";
    my $series = $this->{"series"};
    foreach my $pattern (sort(keys(%$series)))
    {

      my $case = $this->getSerie($pattern);
      my $fetched = $case->{"intermediate"};
      my $prio = $case->{"priority"};
      #$prio = 0 if !$prio;
      #changed introduced alias parsing
      $msg .= "pat[$index]";
      $msg .= "\@".$prio if($prio);
      $msg .= ": ";
      $msg .= "alias " if($case->{"type"} && $case->{"type"} eq 'alias');
      $msg .= "'$pattern'";
      #$msg .= " min=".$case->{"min"} if($case->{"min"});
      $msg .= $case->{"min"} if($case->{"min"});
      #$msg .= "max=".$case->{"max"} if($case->{"max"});
      $msg .= "-".$case->{"max"} if($case->{"max"});
      $msg .= " other: ".join(' ',sort{ $b <=> $a }(keys(%$fetched))) if($fetched && scalar(%$fetched) >0);
      $msg .= "\n";
      $index++;
    }
    $msg .= "---- End of Series ----\n";
  }#else
  #print("returning : $msg\n");
  return($msg);
}# sub series
######################################################################
=pod

=item alias

list/add alias
first argument the alias second the name of the series we want to alias

=cut

######################################################################
sub alias
{
  my  ($this,$line) = @_;

  my $msg;
  my ($pattern,$ref);
  if($line)
  {
    ($pattern,$ref) = split(' ',$line,2);
    $pattern =  $this->patternrevision($pattern);

    my %fetched = ();
    $fetched{"type"} = "alias";

    my $series = $this->{"series"};

    if($series->{$ref})
    {
      if($series->{$ref}->{"type"} && $series->{$ref}->{"type"}  eq "alias")
      {
         $ref = $series->{$ref}->{"ref"};
        $msg .= "LEVE: uhm tryed to make an alias on an alias, redirected to $ref";
      }# if($series->{$ref}->{"type"} && $series->{$ref}->{"type"}  eq "alias")
    }# if(!($series->{$ref}))
    else
    {
        $msg .= "LEVE: uhm trying to setup an alias onto non existing series!";
      return($msg);
    }# else

    $fetched{"ref"} = $ref;

    if($series->{$pattern})
    {
     #uh? the thing allready exists? suppose a manipulation error of the user
     $msg .= "LEVE: a series allready exists with this alias[$pattern] name, choose another alias name!";
    }#/ if($series->{$pattern})
    else
    {
      $series->{$pattern} = \%fetched;
      $msg .= "LEVE: added alias '$pattern' for series $ref";
    }#else
  }# if($pattern)
  else
  {
    my $index = 0;
    $msg .= "---- Aliases ----\n";

    my @aliases = grep (
	($this->{"series"}->{$_}->{"type"} =~ /alias/i), keys(%{$this->{"series"}}));
    #print("Alias command listing:".Dumper(\@aliases)."\n");
    foreach my $entry (@aliases)
    {
      $msg .= "alias[$index]";
      $msg .= ": '$entry' for ";
      $msg .= $this->{"series"}->{$entry}->{"ref"};
      $msg .= "\n";
      $index++;
    }# foreach my $entry (@aliases)
    $msg .= "---- End of aliases ----\n";
  }#else
  #print("returning : $msg\n");
  return($msg);
}# sub alias
######################################################################
=pod

=item request

issue the actual request banner on the ad-channels

=cut

######################################################################
sub request
{
  my  ($this,$line) = @_;
  my $msg = "";

  my $series = $this->{"series"};
  my $settings = $this->{"settings"};
  foreach my $pattern(sort(keys(%$series)))
  {
    my $case = $this->getSerie($pattern);
    if($case->{"priority"} >= $settings->{"ADminpriority"})
    {
      if($case->{"tag"} && $case->{"tag"} ne "")
      {
	$msg .= $case->{"tag"}.",";
      }#if($series->{$pattern}{"tag"} && $series->{$pattern}{"tag"} ne "")
      else
      {
	$pattern =~ s/\.\*\?//g;
	$msg .= $pattern.",";
      }#else
    }#if($series->{$pattern}{"priority"} >= $settings{"ADminpriority"})
  }#if($series->{$pattern})
  if($msg ne "")
  {
    $msg = $settings->{"ADtext"}.": $msg";
  }#if($msg ne "")
  else
  {
    $msg = "No series are marked for advertising";
  }#else

  #$this->print("patternList request, returning: $msg of type ".ref($msg)."\n");
  #print("patternList request, returning: $msg of type ".ref($msg)."\n");
  return $msg;
}#sub request
######################################################################
=pod

=item delseries

delete the series given as parameter, cut and paste from the series
listing

=cut

######################################################################
sub delseries
{
  my  ($this,$pattern) = @_;
  return "-- delseries, usage: delseries pattern\n" if(!$pattern);

  my $series = $this->{"series"};
  $pattern =  $this->patternrevision($pattern);
  $this->delSerie($pattern);
  #delete $series->{$pattern};
  return "LEVE: deleted '$pattern' from series\n";
}#sub delseries
######################################################################
=pod

=item addepisode

in case you fetched an episode by other means, adjust the filter of the
script to drop fetching of that particular episode(s)",

=cut

######################################################################
sub addepisode
{
  my  ($this,$line) = @_;
  my $msg;

  if($line)
  {
    my ($pattern,$rest) = split(' ',$line,2);
    $pattern =  $this->patternrevision($pattern);
    my @newEps = split(' ',$rest);

    #my $series = $this->{"series"};
    my $case = $this->getSerie($pattern);
    #my $mySerie = $series->{$pattern};
    if($case)
    {
      foreach my $episode (@newEps)
      {
	$msg .= "-- adding episode $episode";
	$this->seriesAdvance($pattern,$episode);
      }#foreach my $episode (@newEps)
      $msg .= "-- adding episodes done";
    }#if(%mySerie)
    else
    {
      $msg .= "-- series must exist to add eps!";
    }#else
  }# if($pattern)
  else
  {
    $msg .= "-- add episode, syntax: pattern [list of eps nums]";
  }#else
  return($msg);
}#sub addepisode
######################################################################
=pod

=item delepisode 

in case a fetched episode reveals itself as somehow inappropriated, adjust
the filter of the script to restart fetching of that particular episode(s)",

=cut

######################################################################
sub delepisode 
{ 
  my  ($this,$line) = @_;
  my $msg;

  if($line)
  {
    my ($pattern,$rest) = split(' ',$line,2);
    $pattern =  $this->patternrevision($pattern);
    my @newEps = split(' ',$rest);

    #my $series = $this->{"series"};
    my $case = $this->getSerie($pattern);
    #my $mySerie = $series->{$pattern};
    if(%$case)
    {
      foreach my $episode (@newEps)
      {
	$msg .= "-- resceduling episode $episode\n";
	$this->seriesRescedule($pattern,$episode);
      }#foreach my $episode (@newEps)
      $msg .= "-- resceduling episodes done\n";
    }#if(%mySerie)
    else
    {
      $msg .= "-- series must exist to rescedule eps!\n";
    }#else
  }# if($pattern)
  else
  {
    $msg .= "-- rescedule episode, syntax: pattern [list of eps nums]\n";
  }#else
  return($msg);
}#sub delepisode
######################################################################
=pod

=item seriesRescedule

check series boundaries and adjust them if needed

=cut

######################################################################
sub seriesRescedule
{
  my  ($this,$pattern,$episode) = @_;

  #my $series = $this->{"series"};
    my $case = $this->getSerie($pattern);
  #my $case =  $series->{$pattern};
  my $min = $case->{"min"};
  my $max = $case->{"max"};
  my $fetched = $case->{"intermediate"};

  if ($episode <= $min)
  {
    #actual min wasn't fetched so g oone prior
    $min--;

    #roll up the allready fetched eps up to next gap
    while($min > $episode)
    {
      $fetched->{$min} = 1;
      $min--;
    }#while($case->{$min})

    $case->{"min"} = $episode;
  }#if ($episode == $min+1)
  else
  {
    delete $fetched->{$episode};
  }
}#sub seriesRescedule
######################################################################
=pod

=item modseries

set the importance you give to a given series, mostly determinant for the
request banner generation",

=cut

######################################################################
sub modseries
{
  my  ($this,$line) = @_;
  my ($pattern,$type,$arg) =  split(/ /,$line);
  my $msg = "LEVE: modseries with  '$pattern' '$type' '$arg': all args=".@_."\n";
  $pattern =  $this->patternrevision($pattern);
  $msg .= "LEVE: cleansed up pattern: '$pattern' \n";
  #my $series = $this->{"series"};
    my $case = $this->getSerie($pattern);
  #if($series->{$pattern})
  if($case)
  {
    if($type =~ /tag|prio|max/)
    {
      $case->{"priority"} = $arg if($type eq "prio");
      $case->{"tag"} = $arg if($type eq "tag");
      $case->{"max"} = $arg if($type eq "max");
    }#if($type =~ /tag|prio/)
    else
    {
      $msg .= "you can only modify prio,tag or max\n";
    }#else

  }#if($series->{$pattern})
  else
  {
    $msg .= "usage: tag|prio|max [argument]";
  }#else
  $msg .= "LEVE: updated priority for $pattern";
  return($msg);
}#sub seriesprio
######################################################################
=pod

=item listwishes

list the wishes

=cut

######################################################################
sub listwishes
{
  my  ($this,$pattern) = @_;

  my $msg;
  my $wishlist = $this->{"wishlist"};
  if($pattern)
  {
    $pattern =  $this->patternrevision($pattern);
    push @$wishlist,$pattern;
    $msg .=  "LEVE: added '$pattern' to Wishlist\n";
  }# if($pattern)
  else
  {
    my $index = 0;
    $msg .= "LEVE:---- Wishlist ----\n";
    foreach my $pattern (@$wishlist)
    {
      $msg .= "pat[$index]: '$pattern'\n";
      $index++;
    }
    $msg .= "---- End of Wishlist ----\n";
  }#else
  return($msg);
}#sub listwishes
######################################################################
=pod

=item print

local print method that will seeing the availability launch a POE event or print out manually

=cut

######################################################################
sub print
{
  my  ($this,$msg ) = @_;

  if($this->{"reporter"})
  {
    $this->{"reporter"}->print($msg);
  }# if()
  else
  {
    print("NOHANDLER!!!: $msg\n");
  }# else
}# sub print
######################################################################
=pod

=item rawnames

generate a list of all patterns from the series and the wishlist

=cut

######################################################################
sub rawnames
{
  my($this) = @_;
  my @seriesNames = keys(%{$this->{"series"}});
  my $toreturn = join(' ',@seriesNames)." ".join(' ',$this->{"wishlist"});
  #print("RAWNAMES = $toreturn\n");
  use POE::Helper::Leve::nameContainer;
  my $newcontainer = new POE::Helper::Leve::nameContainer();
  $newcontainer->{"listing"} = $toreturn;

  return $newcontainer;
}#sub rawnames
######################################################################
=pod

=item getSerie

getter for the series hash, since we have plain entries and aliasies, lets have
one place were we handle this difference

=cut

######################################################################
sub getSerie
{
  my($this,$name) = @_;

  my $case =  $this->{"series"}->{$name};
  if($case)
  {
    if($case->{"type"} =~ /alias/i)
    {
      #print("Alias $name detected returning realting instead:".Dumper($this->{"series"}->{$case->{"ref"}})."\n");
      #changed inserted tmpref
      #my $tmpRet =  $this->{"series"}->{$case->{"ref"}};
      #my $tmpRes = { "type" => 'alias' };
      #foreach my $entry (sort(keys(%$tmpRet)))
      #{
      #   #$tmpRes->{$entry} =  $tmpRet->{$entry};
      #   $tmpRes->{$entry} =  \$tmpRet->{$entry};
      #}# foreach my $entry (sort(keys(%$tmpRet)))
      #$tmpRes->{priority} += 1;
      #print("getSeries about to return ".Dumper($tmpRes)."\n");
      #print("getSeries which is copy of".Dumper($tmpRet)."\n");
      #$tmpRes->{priority} -= 1;

      #return $tmpRes;
      return $this->{"series"}->{$case->{"ref"}};
    }# if($case->{"type"} =~ /alias/i)
    return $case; 
  }# if($case)
  return;
}#sub getSerie
######################################################################
=pod

=item delSerie

deleting one series means deleting the real record and all associated aliases

=cut

######################################################################
sub delSerie
{
  my($this,$name) = @_;
  my $case =  $this->{"series"}->{$name};
  if($case)
  {
    if($case->{"type"} =~ /alias/i)
    {
      $name = $case->{"ref"};
    }# if($case->{"type"} =~ /alias/i)
    #delete the original
    delete $this->{"series"}->{$name};
    #now fetch the copies
    my @aliases = grep (
	($this->{"series"}->{$_}->{"type"} =~ /alias/i && 
	 $this->{"series"}->{$_}->{"ref"} eq $name), keys(%{$this->{"series"}}));
    foreach my $entry (@aliases)
    {
      delete $this->{"series"}->{$entry};
      print("Deleted alias $entry\n");
    }# foreach my $entry (@aliases)

  }# if($case)
}#sub delSerie
######################################################################
=pod

=item set

list or modify settings

=cut

######################################################################
sub set
{
  my  ($this,$line) = @_;

  my $msg;
  if($line)
  {
    my ($pattern, $arg) = split(/ /,$line,2);
    if($arg)
    {
      $this->{"settings"}->{$pattern} = $arg;
      $msg .= "LEVE: setting[$line] to '$arg'";
    }# if($arg)
    else
    {
      if($this->{"settings"}->{$pattern})
      {
	$msg .= "LEVE: setting[$pattern]='".$this->{"settings"}->{$pattern}."'";
      }
      else
      {
	$msg .= "LEVE: setting[$pattern] not existent\n";
      }# else
    }#else
  }# if($pattern)
  else
  {
    my $index = 0;
    $msg .= "---- Settings ----\n";
    my $settings = $this->{"settings"};
    foreach my $pattern (sort(keys(%$settings)))
    {

      $msg .= "$pattern = '".$this->{"settings"}->{$pattern}."'";
      $msg .= "\n";
      $index++;
    }
    $msg .= "---- End of Settings ----\n";
  }#else
  #print("returning : $msg\n");
  return($msg);
}# sub set
1
__END__


=back

=head1 THANKS

I. Wronsky came up with this infernal concept and kludged the first 1.x of
leverlada versions together. This gave B. Boettcher the incentive to begin this
work, the famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

